package com.example.mathseduc

import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.OnBackPressedCallback
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Divider
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.example.mathseduc.controllers.ControllerChapter
import com.example.mathseduc.controllers.ControllerLobby
import com.example.mathseduc.controllers.ControllerPlayer
import com.example.mathseduc.controllers.ControllerUtiliser
import com.example.mathseduc.models.Player
import com.example.mathseduc.ui.theme.Colors
import okhttp3.MultipartBody

class ServerDetailsActivity : ComponentActivity() {

    private var playerList: List<Player> = emptyList()
    private val handler = Handler(Looper.getMainLooper())
    private val refreshInterval: Long = 2000

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        val onBackPressedCallback: OnBackPressedCallback = object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
                myBackPressed()
            }
        }

        onBackPressedDispatcher.addCallback(this, onBackPressedCallback)

        setContent {
            ServerDetailPage()
        }
    }

    private fun myBackPressed() {
        val lobbyId = intent.getIntExtra("lobbyId", -1)

        ControllerUtiliser.DeleteUtiliserForLobby(MainActivity.idPlayerConnected, lobbyId)

        if (ControllerLobby.playerCreatorIdPresentInLobby(MainActivity.idPlayerConnected, lobbyId)) {
            val idNextPlayerCreator = ControllerUtiliser.getIdNextPlayerInLobby(lobbyId)

            if (idNextPlayerCreator == -1) {
                ControllerLobby.deleteLobby(lobbyId)
            } else {
                val formDataBuilder = MultipartBody.Builder().setType(MultipartBody.FORM)
                formDataBuilder.addFormDataPart("idplayercreator", idNextPlayerCreator.toString())
                ControllerLobby.updateLobbyIdCreatorLobby(lobbyId, formDataBuilder)
            }
        }
        finish()
    }

    @OptIn(ExperimentalMaterial3Api::class)
    @Composable
    fun ServerDetailPage() {
        val context = LocalContext.current

        val serverName = intent.getStringExtra("serverName")
        val lobbyId = intent.getIntExtra("lobbyId",-1)
        val chapterId = intent.getIntExtra("chapterId",-1)
        val nbPlayers = intent.getIntExtra("nbPlayers",-1)
        val lobbyDifficulty = intent.getIntExtra("lobbyDifficulty",-1)

        var isCreator by rememberSaveable { mutableStateOf(false) }
        var playerListInfos: List<Player> by rememberSaveable { mutableStateOf(emptyList()) }
        var playerList by rememberSaveable { mutableStateOf(ControllerPlayer.getPlayersIdFromLobbyId(lobbyId.toString()) ?: emptyList()) }
        var refreshState by rememberSaveable { mutableStateOf(true) }
        var showDialog by rememberSaveable { mutableStateOf(false) }

        DisposableEffect(refreshState) {
            val handler = Handler(Looper.getMainLooper())
            val refreshRunnable = object : Runnable {
                override fun run() {
                    playerList = ControllerPlayer.getPlayersIdFromLobbyId(lobbyId.toString())!!

                    playerListInfos = playerList.mapNotNull { playerId ->
                        ControllerPlayer.getPlayerInfoById(playerId.toString())
                    }

                    isCreator = ControllerLobby.playerCreatorIdPresentInLobby(MainActivity.idPlayerConnected, lobbyId)

                    if(ControllerLobby.lobbyIsLaunched(lobbyId)){
                        val intent = Intent(context, QuizMultiActivity::class.java)
                        intent.putExtra("serverName", serverName)
                        intent.putExtra("lobbyId", lobbyId)
                        intent.putExtra("chapterId", chapterId)
                        intent.putExtra("nbPlayers", nbPlayers)
                        intent.putExtra("lobbyDifficulty", lobbyDifficulty)
                        context.startActivity(intent)
                        refreshState = false
                    }

                    if (refreshState){
                        handler.postDelayed(this,3000)
                        Log.e("MainActivity", "Refresh ServerDetails")
                    }
                }
            }

            handler.post(refreshRunnable)

            onDispose {
                refreshState = false
                handler.removeCallbacks(refreshRunnable)
            }
        }

        TopAppBar(
                colors = TopAppBarDefaults.topAppBarColors(
                        containerColor = Color.Transparent,
                ),
                title = {},
                navigationIcon = {
                    IconButton(
                            onClick = { showDialog = true },
                            modifier = Modifier.size(60.dp)
                    ) {
                        Icon(
                                imageVector = Icons.Filled.ArrowBack,
                                contentDescription = "Retour",
                                modifier = Modifier.size(36.dp),
                                tint = Color.White
                        )
                    }
                },
        )

        val modifier = Modifier
                .fillMaxSize()
                .padding(16.dp)

        Column(
            modifier = modifier,
            verticalArrangement = Arrangement.Center,
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Text(
                text = serverName!!,
                color = Color.White,
                    fontSize = 30.sp,
                fontWeight = FontWeight.Bold
            )

            Spacer(modifier = Modifier.height(25.dp))

                Text(
                        text = "Lobby Settings",
                        fontSize = 23.sp,
                        color = Color.White,
                        fontWeight = FontWeight.Bold
                )
                Text(
                        text = "Chapter : ${ControllerChapter.getChapterNameById(chapterId).toString()}",
                        fontSize = 20.sp,
                        color = Color.White
                )
                Text(
                        text = "Players : ${ControllerLobby.getNbPlayerInLobby(lobbyId)}/${nbPlayers}",
                        fontSize = 20.sp,
                        color = (if(ControllerLobby.getNbPlayerInLobby(lobbyId)==nbPlayers) Color.Red else Color.White)
                )

                Text(
                        text = "Difficulty : $lobbyDifficulty",
                        fontSize = 20.sp,
                        color = Colors.White,
                )
            

            Spacer(modifier = Modifier.height(30.dp))

            Text(
                    text = "Player Name",
                    fontSize = 20.sp,
                    color = Colors.White,
            )
            Divider(
                    color = Color.Gray,
                    thickness = 2.dp,
                    modifier = Modifier.fillMaxWidth()
            )
            LazyColumn(
                modifier = Modifier
                        .fillMaxSize()
                        .weight(0.75f)
                        .padding(top = 2.dp),
                    horizontalAlignment = Alignment.CenterHorizontally
            ) {
                items(playerListInfos) { player ->
                    Text(
                        text = player.nickname,
                        fontSize = 18.sp,
                        color = Colors.White,
                        modifier = Modifier.weight(1f)
                                .padding(top = 5.dp)
                    )
                }
            }
            if (isCreator) {
                val formDataBuilder = MultipartBody.Builder().setType(MultipartBody.FORM)
                formDataBuilder.addFormDataPart("launched", "1")
                Button(
                    onClick = {
                        ControllerLobby.updateLobbyLauched(lobbyId,formDataBuilder)
                    },
                    shape = RoundedCornerShape(15),
                    enabled = isCreator,
                    colors = ButtonDefaults.buttonColors(Colors.Green),
                    modifier = Modifier
                            .fillMaxWidth()
                            .height(48.dp)
                ) {
                    Text(
                        text = "LAUNCH",
                        color = Color.White,
                        fontWeight = FontWeight.Bold
                    )
                }
            }
            if (showDialog) {
                LeaveLobbyDialog(serverName, onConfirmLeave = {},onCancelLeave = { showDialog = false })
            }
        }
    }
    @Composable
    fun LeaveLobbyDialog(namelobby: String, onConfirmLeave: () -> Unit, onCancelLeave: () -> Unit) {
        val context = LocalContext.current

        AlertDialog(
                onDismissRequest = onCancelLeave,
                title = { Text("Confirm leaving lobby") },
                confirmButton = {
                    Button(
                            onClick = {
                                onConfirmLeave()
                            }
                    ) {
                        Text("Leave")
                    }
                },
                dismissButton = {
                    Button(
                            onClick = {
                                onCancelLeave()
                            }
                    ) {
                        Text("Cancel")
                    }
                },
                text = {
                    Text("Are you sure you want to leave the lobby $namelobby?")
                }
        )
    }

}
